import unittest

from etlstat.security import password

min_length = 8
max_length = 16


class TestPassword(unittest.TestCase):

    def testGeneratePasswordOk(self):
        pwd = password.generate_password(min_length, max_length, first_letter=True)
        self.assertTrue(len(pwd) >= min_length)
        self.assertTrue(len(pwd) <= max_length)
        self.assertTrue(pwd[:1].isalpha())

    def testGeneratePasswordFail(self):
        try:
            password.generate_password(max_length, min_length)
        except TypeError as e:
            self.assertEqual(str(e), 'Minimum password length must be less or equal than maximum.')

if __name__ == '__main__':
    unittest.main()



# coding: utf-8
"""
    MySQL

    Date:
        24/01/2018

    Author:
        slave110

    Version:
        Alpha

    Notes:


"""

import numpy as np
import os
from pandas import DataFrame, isnull
import re
from sqlalchemy import create_engine
from sqlalchemy.exc import DatabaseError


class MySQL:

    engine = None

    # TODO: check this conversion map
    conversion_map = {
        'object': 'VARCHAR(255)',
        'int32': 'INT',
        'int64': 'INT',
        'float32': 'DECIMAL(20,6)',
        'float64': 'DECIMAL(20,6)'
    }

    def __init__(self, user, password, host, port, database):
        conn_string = "mysql+mysqlconnector://{0}:{1}@{2}:{3}/{4}".format(
            user,
            password,
            host,
            port,
            database)
        self.engine = create_engine(conn_string)

    def check_for_table(self, table, schema=None):
        """
        Check if table exists in database.

        :param table: (:obj:`str`): Database table's name.
        :param schema: (:obj:`str`): Schema name
        Returns:
            bool: True if table exists, False in otherwise.
        """
        return self.engine.dialect.has_table(self.engine, table, schema=schema)

    def execute_sql(self, sql):
        """
        Executes a DDL or DML SQL statement
        :param sql: SQL statement
        :return: status (True | False); result (data frame or error string)
        """
        connection = self.engine.connect()
        try:
            result = connection.execute(sql)
            status = True
            if re.match('^[ ]*SELECT .*', sql, re.IGNORECASE):
                rows = result.fetchall()
                result = DataFrame(rows, columns=result.keys())
        except DatabaseError as e:
            result = e
            status = False
        finally:
            connection.close()
        return status, result

    def create(self, table):
        """
        Create a new table in database from DataFrame format.

        :param table: DataFrame which name and column's label match with database
        table's name and columns that you wish to create.
        """
        connection = self.engine.connect()

        if not self.check_for_table(table.name):
            sql = "CREATE TABLE"

            if isinstance(table, DataFrame):
                sql += " `{0}` (".format(table.name)

                for label in table:
                    sql += "`{0}` {1}, ".format(label, self.conversion_map[str(table[label].dtype)])
                sql = sql[:-2] + ')'

                try:
                    rts = connection.execute(sql)
                    rts.close()
                    status = True
                except DatabaseError as e:
                    status = False
                finally:
                    connection.close()

                return status

        return False

    def select(self, table, conditions=''):
        """
        Select data from table.

        :param table: (:obj:`str` or :obj:`DataFrame`): Table's name in database if
                    you want read all fields in database table or a DataFrame
                    which name and column's label match with table's name and
                    columns table that you want read from database.
        :param conditions: (:obj:`str` or :obj:`list` of :obj:`str`, optional):
                    A select condition or list of select conditions with sql
                    syntax.
        Returns:
            :obj:`DataFrame`: A DataFrame with data from database.

        """
        connection = self.engine.connect()

        df = None
        sql = "SELECT"
        if isinstance(table, str):
            sql += " {0} FROM `{1}`".format('*', table)
        elif isinstance(table, DataFrame):
            if len(list(table)) > 0:
                for label in list(table):
                    sql += " {0},".format(label)
            else:
                sql += " *  "

            sql = sql[:-1]
            sql += " FROM `{0}`".format(table.name)
        else:
            raise TypeError("table must be a string or DataFrame.")

        if isinstance(conditions, str) and conditions is not '':
            sql += " WHERE {0}".format(conditions)
        elif isinstance(conditions, list) and len(conditions) > 0:
            sql += " WHERE"
            for condition in conditions:
                sql += " {0},".format(condition)
            sql = sql[:-1]

        try:
            rts = connection.execute(sql)
            df = None
            if rts.rowcount > 0:
                df = DataFrame(rts.fetchall())
                df.columns = rts.keys()
            rts.close()
        except DatabaseError as e:
            print(e)
        finally:
            connection.close()

        return df

    def insert(self, table, rows=None):
        """
        Insert DataFrame's rows in a database table.

        :param table: (:obj:`DataFrame`): DataFrame which name and column's label
                    match with table's name and column's name in database. It
                    must filled with data rows.
        :param rows: (:obj:`list` of int, optional): A list of row's indexes that you
                    want insert to database.
        Returns:
            int: number of rows matched.
        """
        rows_matched = 0

        connection = self.engine.connect()

        if isinstance(table, DataFrame):
            if not self.check_for_table(table.name):
                self.create(table)

            sql = "INSERT INTO"
            sql += " `{0}`".format(table.name)
            sql += ' ('
            for label in list(table):
                sql += "{0}, ".format(label)
            sql = sql[:-2]
            sql += ')'

            for index, row in table.iterrows():

                if rows is None or index in rows:
                    sql_insert = sql
                    sql_insert += " VALUES ("
                    for value in row:
                        if isinstance(value, str):
                            sql_insert += "'{0}', ".format(value)
                        else:
                            if isnull(value):
                                sql_insert += "{0}, ".format('NULL')
                            else:
                                sql_insert += "{0}, ".format(value)
                    sql_insert = sql_insert[:-2] + ')'

                    rts = connection.execute(sql_insert)
                    rows_matched += rts.rowcount

            rts.close()
            connection.close()

        else:
            raise TypeError("table must be a DataFrame.")

        return rows_matched

    def update(self, table, index=None):
        """
        Update rows in a database table.

        :param table: (:obj:`DataFrame`): DataFrame which name and column's label
                    match with table's name and columns name in database. It must
                    be filled with data rows.
        :param index: (:obj:`list` of name columns): list of DataFrame's columns names
                    use as index in the update search. Other columns will be
                    updated in database.
        Returns:
            int: number of rows matched.
        """
        rows_matched = 0

        connection = self.engine.connect()

        if isinstance(table, DataFrame):
            if isinstance(index, list):
                for row in table.values:
                    sql = "UPDATE `{0}` SET".format(table.name)
                    sql_conditions = ''
                    sql_updates = ''
                    for id, label in enumerate(table):
                        if label not in index:
                            if isinstance(row[id], str):
                                sql_updates += " %s='%s'," % (label, row[id])
                            else:
                                sql_updates += " {0}={1},".format(label, row[id])
                        else:
                            if isinstance(row[id], str):
                                sql_conditions += " %s='%s' and" % (label, row[id])
                            else:
                                sql_conditions += " {0}={1} and".format(label, row[id])
                    sql += sql_updates[:-1]

                    if len(sql_conditions) > 1:
                        sql += ' WHERE' + sql_conditions[:-4]

                    rts = connection.execute(sql)  # ResultProxy
                    rows_matched += rts.rowcount
                    rts.close()

                connection.close()
        else:
            raise TypeError("table must be a DataFrame.")

        return rows_matched

    def delete(self, table, conditions=''):
        """
        Delete data from table.

        Args:
        :param table: (:obj:`str`): Database table name that you wish delete rows.
        :param conditions: (:obj:`str`, optional): A string of select conditions
            with sql syntax.
        Returns:
            int: number of rows matched.
        """
        connection = self.engine.connect()

        sql = "DELETE FROM `{0}`".format(table)

        if isinstance(conditions, str) and conditions is not '':
            sql += ' WHERE ' + conditions

        try:
            rts = connection.execute(sql)
            rows_matched = rts.rowcount
            rts.close()
        except DatabaseError as e:
            rows_matched = None
            print(e)
        finally:
            connection.close()

        return rows_matched

    def bulk_insert(self, table, csv_path='temp.csv', sep=';', header=False, index=False):
        """
        Make a bulk insert in database.

        :param table: (:obj:`DataFrame`): DataFrame which name and column's label
                    match with table's name and columns in database. It must be
                    filled with data rows.
        :param csv_path: (:str): default csv file
        :param sep: (:str): default field separator
        :param header: (:bool): default header row is present or not
        :param index: (:bool): default write or not row names
        Returns:
            int: number of rows matched.
        """
        connection = self.engine.connect()

        if not self.check_for_table(table.name):
            self.create(table)

        if isinstance(table, DataFrame):
            aux = table.replace(np.NaN, "\\N")
            aux.to_csv(csv_path, sep=sep, header=header, index=index)
        else:
            raise TypeError("table must be a DataFrame.")

        sql = """LOAD DATA LOCAL INFILE '{0}' INTO TABLE `{1}` FIELDS TERMINATED BY ';' ENCLOSED BY '"' """\
              .format(csv_path, table.name)

        try:
            rts = connection.execute(sql)
            rows_matched = rts.rowcount
            rts.close()
        except DatabaseError as e:
            rows_matched = None
            print(e)
        finally:
            connection.close()

        os.remove(csv_path)

        return rows_matched



import csv
from etlstat.extractor.pcaxis import *
import config_test as ct
import unittest


class TestPcaxis(unittest.TestCase):

    base_path = ct.GLOBAL_PATH + '/etlstat/extractor/test/'

    def testUriType(self):
        self.assertEqual(uri_type('2184.px'), 'FILE', 'Uri type differs!')
        self.assertEqual(uri_type('http://www.ine.es/jaxiT3/files/t/es/px/2184.px'), 'URL', 'Uri type differs!')

    def testMetaDataSplit(self):
        pc_axis = """AXIS-VERSION="2006";
        CREATION-DATE="20170913";
        CHARSET="ANSI";
        SUBJECT-AREA="Indice de Precios de Consumo. Base 2016";
        SUBJECT-CODE="null";
        MATRIX="148";
        TITLE="Índices por comunidades autónomas: general y de grupos ECOICOP";
        CONTENTS="Índices";
        CODEPAGE="iso-8859-15";
        DESCRIPTION="";
        COPYRIGHT=YES;
        DECIMALS=1;
        SHOWDECIMALS=1;
        STUB="Comunidades y Ciudades Autónomas","Grupos ECOICOP";
        HEADING="Tipo de dato","Periodo";
        VALUES("Comunidades y Ciudades Autónomas")="Nacional","Andalucía","Aragón","Asturias, Principado de","Balears, Illes","Canarias",
        "Cantabria","Castilla y León","Castilla - La Mancha","Cataluña","Comunitat Valenciana",
        "Extremadura","Galicia","Madrid, Comunidad de","Murcia, Región de",
        "Navarra, Comunidad Foral de","País Vasco","Rioja, La","Ceuta","Melilla";
        VALUES("Grupos ECOICOP")="00 Índice general","01 Alimentos y bebidas no alcohólicas",
        "02 Bebidas alcohólicas y tabaco","03 Vestido y calzado",
        "04 Vivienda, agua, electricidad, gas y otros combustibles",
        "05 Muebles, artículos del hogar y artículos para el mantenimiento corriente del hogar",
        "06 Sanidad","07 Transporte","08 Comunicaciones","09 Ocio y cultura","10 Enseñanza",
        "11 Restaurantes y hoteles","12 Otros bienes y servicios ";
        VALUES("Tipo de dato")="Índice","Variación mensual","Variación anual","Variación en lo que va de año";
        VALUES("Periodo")="2017M08","2017M07","2017M06","2017M05","2017M04","2017M03","2017M02","2017M01",
        "2016M12","2016M11","2016M10","2016M09","2016M08","2016M07","2016M06","2016M05","2016M04",
        "2016M03","2016M02","2016M01","2015M12","2015M11","2015M10","2015M09","2015M08","2015M07",
        "2015M06","2015M05","2015M04","2015M03","2015M02","2015M01","2014M12","2014M11","2014M10",
        "2014M09","2014M08","2014M07","2014M06","2014M05","2014M04","2014M03","2014M02","2014M01",
        "2013M12","2013M11","2013M10","2013M09","2013M08","2013M07","2013M06","2013M05","2013M04",
        "2013M03","2013M02","2013M01","2012M12","2012M11","2012M10","2012M09","2012M08","2012M07",
        "2012M06","2012M05","2012M04","2012M03","2012M02","2012M01","2011M12","2011M11","2011M10",
        "2011M09","2011M08","2011M07","2011M06","2011M05","2011M04","2011M03","2011M02","2011M01",
        "2010M12","2010M11","2010M10","2010M09","2010M08","2010M07","2010M06","2010M05","2010M04",
        "2010M03","2010M02","2010M01","2009M12","2009M11","2009M10","2009M09","2009M08","2009M07",
        "2009M06","2009M05","2009M04","2009M03","2009M02","2009M01","2008M12","2008M11","2008M10",
        "2008M09","2008M08","2008M07","2008M06","2008M05","2008M04","2008M03","2008M02","2008M01",
        "2007M12","2007M11","2007M10","2007M09","2007M08","2007M07","2007M06","2007M05","2007M04",
        "2007M03","2007M02","2007M01","2006M12","2006M11","2006M10","2006M09","2006M08","2006M07",
        "2006M06","2006M05","2006M04","2006M03","2006M02","2006M01","2005M12","2005M11","2005M10",
        "2005M09","2005M08","2005M07","2005M06","2005M05","2005M04","2005M03","2005M02","2005M01",
        "2004M12","2004M11","2004M10","2004M09","2004M08","2004M07","2004M06","2004M05","2004M04",
        "2004M03","2004M02","2004M01","2003M12","2003M11","2003M10","2003M09","2003M08","2003M07",
        "2003M06","2003M05","2003M04","2003M03","2003M02","2003M01","2002M12","2002M11","2002M10",
        "2002M09","2002M08","2002M07","2002M06","2002M05","2002M04","2002M03","2002M02","2002M01";
        CODES("Comunidades y Ciudades Autónomas")="null","CA01","CA02","CA03","CA04","CA05","CA06","CA07","CA08","CA09",
        "CA10","CA11","CA12","CA13",
        "CA14","CA15","CA16","CA17","CA18","CA19";
        MAP("Comunidades y Ciudades Autónomas")="spain_regions_img_ind";
        PRECISION("Tipo de dato","Índice")=3;
        UNITS="  Índice,   Tasas";
        SOURCE="Instituto Nacional de Estadística";
        DATA=
        101.553 101.351 102.055 102.011 102.073 101.101 101.122 101.488 102.049 101.447
        """
        meta, data = meta_data_split(pc_axis)
        self.assertEqual(meta[13], 'STUB="Comunidades y Ciudades Autónomas","Grupos ECOICOP"', 'Metadata differs!')
        self.assertEqual(meta[14], 'HEADING="Tipo de dato","Periodo"', 'Metadata differs!')
        self.assertEqual(meta[22], 'UNITS="  Índice,   Tasas"', 'Metadata differs!')
        meta_dict = meta_split(meta)
        self.assertEqual(meta_dict['VALUES(Comunidades y Ciudades Autónomas)'][2], 'Aragón',
                         'Metadata differs!')
        self.assertEqual(meta_dict['VALUES(Comunidades y Ciudades Autónomas)'][4], 'Balears, Illes',
                         'Metadata differs!')

    def testDimensions(self):
        pc_axis = """AXIS-VERSION="2006";
        CREATION-DATE="20170526";
        CHARSET="ANSI";
        SUBJECT-AREA="Encuesta de financiación y gastos de la enseñanza privada. Curso 2004-2005";
        SUBJECT-CODE="null";
        MATRIX="01001";
        TITLE="Principales resultados estructurales por Tipo de indicador y Nivel educativo (agregado)";
        CONTENTS="Principales resultados estructurales";
        CODEPAGE="iso-8859-15";
        DESCRIPTION="";
        COPYRIGHT=YES;
        DECIMALS=0;
        SHOWDECIMALS=0;
        STUB="Tipo de indicador";
        HEADING="Nivel educativo (agregado)";
        VALUES("Tipo de indicador")="Número de Centros","Número de Alumnos","Personal remunerado con tareas docentes",
        "Personal remunerado con tareas no docentes","Total de Personal Remunerado",
        "Total de Personal no Remunerado","Total de Personal";
        VALUES("Nivel educativo (agregado)")="TOTAL","Educación no Universitaria","Educación Universitaria";
        UNITS="valores absolutos";
        SOURCE="Instituto Nacional de Estadística";
        DATA=
        6621.0 6477.0 144.0 2478366.0 2288630.0 189736.0 181347.0 170919.0 10428.0 44702.0 39962.0 4740.0 225724.0 210556.0 
        15168.0 6790.0 5949.0 841.0 232708.0 216505.0 16203.0;
        """
        meta, data = meta_data_split(pc_axis)
        meta_dict = meta_split(meta)
        dimension_names, dimension_members = dimensions(meta_dict)
        self.assertEqual(dimension_names[0], 'Tipo de indicador', 'Dimension name differs!')
        self.assertEqual(dimension_names[1], 'Nivel educativo (agregado)', 'Dimension name differs!')
        self.assertEqual(dimension_members[0][0], 'Número de Centros', 'Dimension member differs!')
        self.assertEqual(dimension_members[0][1], 'Número de Alumnos', 'Dimension member differs!')

    def testFromPcaxis(self):
        pc_axis_dict = from_pc_axis(self.base_path + 'px/2184.px', encoding='ISO-8859-2')
        self.assertEqual(pc_axis_dict['DATA'].dtypes['DATA'], 'float64')
        self.assertEqual(pc_axis_dict['METADATA']['VALUES(Índices y tasas)'].index('Índice'), 0, 'Dictionary index differs!')
        self.assertGreater(len(pc_axis_dict['DATA']), 9500)

    def testToCsv(self):
        # fichero 1
        pc_axis_dict = from_pc_axis('http://www.ine.es/jaxiT3/files/es/2184.px', encoding='windows-1252')
        pc_axis_dict['DATA'].to_csv(
            path_or_buf=self.base_path + 'px/2184.csv',
            sep=',',
            header=True,
            index=False,
            doublequote=True,
            quoting=csv.QUOTE_NONNUMERIC,
            encoding='utf-8')
        fd = open(self.base_path + 'px/2184.csv', 'r')
        for line in fd:
            pass
        last = line
        fd.close()
        self.assertEqual(last, '"19 Melilla","Vivienda segunda mano","Variación en lo que va de año","2007T1",""\n')
        # fichero 2
        pc_axis_dict = from_pc_axis('http://www.ine.es/jaxiT3/files/es/1443.px', encoding='windows-1252')
        pc_axis_dict['DATA'].to_csv(
            path_or_buf=self.base_path + 'px/1443.csv',
            sep=',',
            header=True,
            index=False,
            doublequote=True,
            quoting=csv.QUOTE_NONNUMERIC,
            encoding='utf-8')
        fd = open(self.base_path + 'px/1443.csv', 'r')
        for line in fd:
            pass
        last = line
        fd.close()
        print(last)
        self.assertEqual(last, '"19 Melilla","Extranjera","1975",""\n')

    def testHTTPError(self):
        # returns status code 404
        url = 'http://www.ine.es/jaxi'
        pc_axis_dict = from_pc_axis(url, encoding='windows-1252')
        self.assertEqual(len(pc_axis_dict['DATA']), 1)

    def testConnectionError(self):
        # raises a timeout error
        url = 'http://www.ine.net/jaxiT3/files/t/es/px/1001.px'
        pc_axis_dict = from_pc_axis(url, encoding='windows-1252')
        self.assertEqual(len(pc_axis_dict['DATA']), 1)





# coding: utf-8
"""
    SimpleSQL

    Date:
        02/03/2018

    Author:
        slavebuffer

    Version:
        Alpha

    Notes:


"""

from etlstat.database.mysql import MySQL


class SimpleSQL:

    @classmethod
    def parse_conn_string(cls, config):
        """
        Split conn_string, acquired from Configuration object into db_params and
        database_name.

        Args:
            config (Configuration): object Configuration filled with all params
                                    needed.
        Returns:
            db_params (str[]): user, password, host and port terms into list.
            database_name (str): name of database schema.
        """
        conn_string = config.store.conn_string
        connector, conn_data = conn_string.split('//')
        user, link, port_db = conn_data.split(':')
        port, database_name = port_db.split('/')
        password, host = link.split('@')

        db_params = None
        if connector == 'mysql+mysqlconnector:':
            db_params = [user, password, host, port]
        else:
            raise NotImplementedError("Not implemented driver for this connection!")

        return db_params, database_name

    @classmethod
    def push(cls, table, config):
        """
        Insert dataframe's data into database table. If table doesn't exist, push
        will create it.

        Args:
            table (DataFrame): a pandas DataFrame object filled with data that you
                               want insert into database.
            config (Configuration): object Configuration filled with all params
                                    needed.
        """
        db_params, database_name = cls.parse_conn_string(config)

        conn = MySQL(*db_params, database_name)

        if not conn.check_for_table(table.name):
            conn.create(table)
            conn.execute_sql("ALTER TABLE %s ADD COLUMN `id` INT PRIMARY KEY AUTO_INCREMENT" % table.name)
            if len(table) > 0 and len(table) < 10:
                conn.insert(table)
            elif len(table) > 9:
                conn.bulk_insert(table)
        else:
            conn.insert(table)

    @classmethod
    def pull(cls, table, config):
        """
        Request data columns dataframe from database.

        Args:
            table (DataFrame): a empty pandas DataFrame object structured
                               with data columns.
            config (Configuration): object Configuration filled with all params
                                    needed.
        Returns:
            result (DataFrame): a pandas DataFrame filled with data from database.
        """
        db_params, database_name = cls.parse_conn_string(config)

        conn = MySQL(*db_params, database_name)

        result = None
        if isinstance(table, str):
            if conn.check_for_table(table):
                result = conn.select(table)
        else:
            if conn.check_for_table(table.name):
                result = conn.select(table)

        return result

    @classmethod
    def drop(cls, table_name, config):
        """
        Drop database table called table_name.

        Args:
            table_name (str): database table name.
            config (Configuration): object Configuration filled with all params
                                    needed.
        """
        db_params, database_name = cls.parse_conn_string(config)

        conn = MySQL(*db_params, database_name)

        if conn.check_for_table(table_name):
            conn.execute_sql("DROP TABLE %s" % table_name)


# -*- coding: utf-8 -*-

"""Pcaxis Parser module

This module obtains a pandas DataFrame of tabular data from a PC-Axis file or URL.
Reads data and metadata from PC-Axis into a dataframe and dictionary, and returns a
dictionary containing both structures.

Example:
    from etlstat.extractor.pcaxis import *

    dict = from_pc_axis(self.base_path + 'px/2184.px', encoding='ISO-8859-2')
    
References:
    PX-file format specification AXIS-VERSION 2013:
        https://www.scb.se/Upload/PC-Axis/Support/Documents/PX-file_format_specification_2013.pdf

Todo:
    meta_split: "NOTE" attribute can be multiple, but only the last one is added to the dictionary
"""

import itertools
import numpy
import pandas
import re
import requests
from etlstat.log.timing import timeit, log

log.basicConfig(level=log.ERROR)
logger = log.getLogger(__name__)


def uri_type(uri):
    """
    Determines the type of URI.

    Args:
        uri (str): pc-axis file name or URL

    Returns:
        str_type (str): 'URL' | 'FILE'

    ..  Regex debugging:
        https://pythex.org/
    """
    str_type = 'FILE'
    if re.match('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', uri):
        str_type = 'URL'

    return str_type


@timeit
def read(uri, encoding, timeout=10):
    """
    Reads a text file from file system or URL.

    Args:
        uri (str): file name or URL
        encoding (str): charset encoding
        timeout (int): request timeout; optional

    Returns:
        pc_axis (str): file contents.
    """

    pc_axis = """
        STUB="";
        HEADING="";
        VALUES("")=""
        DATA=".."
        """

    if uri_type(uri) == 'URL':
        try:
            response = requests.get(uri, stream=True, timeout=timeout)
        except requests.exceptions.ConnectionError as connection_error:
            logger.error('ConnectionError = ' + str(connection_error))
        except requests.exceptions.HTTPError as http_error:
            logger.error('HTTPError = ' + http_error.response.status_code + ' ' + http_error.response.reason)
        except requests.exceptions.InvalidURL as url_error:
            logger.error('URLError = ' + url_error.response.status_code + ' ' + url_error.response.reason)
        except Exception:
            import traceback
            logger.error('Generic exception: ' + traceback.format_exc())
            raise
        finally:
            if 'response' in locals():
                if response.status_code == 200:
                    response.encoding = encoding
                    pc_axis = response.text
                response.close()
    else:
        file_object = open(uri, encoding=encoding)
        pc_axis = file_object.read()
        file_object.close()

    return pc_axis


def meta_data_split(pc_axis):
    """
    Extracts metadata and data from pc-axis file contents.

    Args:
        pc_axis (str): pc_axis file contents.

    Returns:
        meta (list of string): each item conforms to pattern ATTRIBUTE=VALUES
        data (string): data values
    """
    # replace new line characters with blank
    pc_axis = pc_axis.replace('\n', ' ').replace('\r', ' ')

    # split file into metadata and data sections
    meta_data, data = pc_axis.split('DATA=')
    # meta: list of strings that conforms to pattern ATTRIBUTE=VALUES
    meta = re.findall('([^=]+=[^=]+)(?:;|$)', meta_data)
    # remove trailing blanks and final semicolon
    data = data.strip().rstrip(';')
    for i, item in enumerate(meta):
        meta[i] = item.strip().rstrip(';')

    return meta, data


def meta_split(meta_list):
    """
    Splits the list of metadata elements into a dictionary of multi-valued keys.

    Args:
        meta_list (list of string): pairs ATTRIBUTE=VALUES

    Returns:
        meta_dict (dictionary): {'attribute1': ['value1', 'value2', ... ], ...}
    """
    meta_dict = {}

    for m in meta_list:
        name, values = m.split('=')
        # remove double quotes from key
        name = name.replace('"', '')
        # split values delimited by double quotes into list
        # additionally strip leading and trailing blanks
        v_list = re.findall('"[ ]*(.+?)[ ]*"+?', values)
        meta_dict[name] = v_list

    return meta_dict


def dimensions(meta_dict):
    """
    Reads STUB and HEADING values from metadata dictionary.

    Args:
        meta_dict: dictionary of metadata

    Returns:
        dimension_names (list)
        dimension_members (list)
    """

    dimension_names = []
    dimension_members = []

    # add STUB and HEADING elements to a list of dimension names
    st = meta_dict['STUB']
    for s in st:
        dimension_names.append(s)
    hd = meta_dict['HEADING']
    for h in hd:
        dimension_names.append(h)

    # add VALUES of STUB and HEADING to a list of dimension members
    st = meta_dict['STUB']
    for s in st:
        s_val = []
        values = meta_dict['VALUES(' + s + ')']
        for v in values:
            s_val.append(v)
        dimension_members.append(s_val)

    # add HEADING values to the list of dimension members
    hd = meta_dict['HEADING']
    for h in hd:
        h_val = []
        values = meta_dict['VALUES(' + h + ')']
        for v in values:
            h_val.append(v)
        dimension_members.append(h_val)

    return dimension_names, dimension_members


def to_data_frame(dimension_names, dimension_members, data_list):
    """
    Builds a data frame by adding the cartesian product of dimension members,
    plus series of data.

    Args:
        dimension_names (list of string)
        dimension_members (list of string)
        data_list (list of string)

    Returns:
        data_frame (pandas data frame)
    """

    # cartesian product of dimension members
    dim_exploded = list(itertools.product(*dimension_members))

    data_frame = pandas.DataFrame(data=dim_exploded, columns=dimension_names)

    # convert data values from string to float
    for index, value in enumerate(data_list):
        try:
            data_list[index] = float(value)
        except ValueError:
            data_list[index] = numpy.nan

    # column of data values
    data_frame['DATA'] = pandas.Series(data_list)

    return data_frame


@timeit
def from_pc_axis(uri, encoding, timeout=10):
    """
    Extracts metadata and data sections from pc-axis.

    Args:
        uri (str): file name or URL
        encoding (str): charset encoding
        timeout (int): request timeout in seconds; optional

    Returns:
         pc_axis_dict (dictionary): dictionary of metadata and pandas data frame
    """

    # get file content or URL stream
    pc_axis = read(uri, encoding, timeout)

    # metadata and data extraction and cleaning
    meta, data = meta_data_split(pc_axis)

    # stores raw metadata into a dictionary
    meta_dict = meta_split(meta)

    # explode raw data into a list of float values
    data_list = data.split()

    # extract dimension names and members from 'meta_dict' STUB and HEADING keys
    dimension_names, dimension_members = dimensions(meta_dict)

    # build a data frame
    data_frame = to_data_frame(dimension_names, dimension_members, data_list)

    # dictionary of metadata and data
    pc_axis_dict = {
        'METADATA': meta_dict,
        'DATA': data_frame
    }
    return pc_axis_dict



# coding: utf-8
"""
    Oracle

    Date:
        23/01/2018

    Author:
        lla11358

    Version:
        Alpha

    Notes:


"""

import csv
from pandas import DataFrame, isnull
import os
import re
from sqlalchemy import create_engine
from sqlalchemy.exc import DatabaseError


class Oracle:

    engine = None

    # TODO: check this conversion map
    conversion_map = {
        'object': 'VARCHAR2(256)',
        'int32': 'INTEGER',
        'int64': 'INTEGER',
        'float32': 'NUMBER',
        'float64': 'NUMBER'
    }

    def __init__(self, user, password, host, port, service_name):
        conn_string = "oracle+cx_oracle://{0}:{1}@{2}:{3}/?service_name={4}".format(
            user,
            password,
            host,
            port,
            service_name)
        self.engine = create_engine(
            conn_string,
            encoding='utf8',
            coerce_to_unicode=True,
            coerce_to_decimal=False)

    def check_for_table(self, table, schema=None):
        """
        Check if table exists in database.

        Args:
            table (str): Database table's name.
            schema (str): Database schema's name.

        Returns:
            bool: True if table exists, False in otherwise.
        """
        return self.engine.dialect.has_table(self.engine, table, schema=schema)

    def execute_sql(self, sql):
        """
        Executes a DDL or DML SQL statement.

        Args:
            sql (str): SQL statement

        Returns:
            status (bool)
            result (data frame or error)
        """
        connection = self.engine.connect()
        try:
            result = connection.execute(sql)
            status = True
            if re.match('^[ ]*SELECT .*', sql, re.IGNORECASE):
                rows = result.fetchall()
                result = DataFrame(rows, columns=result.keys())
        except DatabaseError as e:
            result = e
            status = False
        finally:
            connection.close()
        return status, result

    def create(self, table, schema=None):
        """
        Create a new table in database from DataFrame format.

        Args:
            table (DataFrame): its name and column's label match with database table.
            schema (str): database schema. If None, then the table is created in the
            user's default schema.
        """
        connection = self.engine.connect()

        if not self.check_for_table(table.name, schema=schema):
            sql = "CREATE TABLE "
            if schema:
                sql += "{0}.".format(schema)

            if isinstance(table, DataFrame):
                sql += "{0} (".format(table.name)

                for label in table:
                    sql += "{0} {1}, ".format(label, self.conversion_map[str(table[label].dtype)])
                sql = sql[:-2] + ')'
                try:
                    rts = connection.execute(sql)
                    rts.close()
                    status = True
                except DatabaseError as e:
                    print(e)
                    status = False
                finally:
                    connection.close()

                return status

        return False

    def select(self, table, schema=None, conditions=''):
        """
        Select data from table.

        Args:
            table: (obj str or obj DataFrame): Table's name in database if
            you want read all fields in database table, or a DataFrame whose
            name and column's label match with table's name and columns table
            that you want to read from database.
            schema (str): database schema
            conditions (str or list): optional. A select condition or list of
            select conditions with sql syntax.

        Returns:
            df (obj DataFrame): a DataFrame with data from database.

        """
        connection = self.engine.connect()

        df = None
        sql = "SELECT "

        if schema:
            sql += "{0}.".format(schema)

        if isinstance(table, str):
            sql += "{0} FROM {1}".format('*', table)
        elif isinstance(table, DataFrame):
            for label in list(table):
                sql += " {0},".format(label)

            sql = sql[:-1]
            sql += " FROM {0}".format(table.name)
        else:
            raise TypeError("table must be a string or DataFrame.")

        if isinstance(conditions, str) and conditions is not '':
            sql += " WHERE {0}".format(conditions)
        elif isinstance(conditions, list) and len(conditions) > 0:
            sql += " WHERE"
            for condition in conditions:
                sql += " {0},".format(condition)
            sql = sql[:-1]

        try:
            rts = connection.execute(sql)
            df = DataFrame(rts.fetchall(), columns=rts.keys())
            rts.close()
        except DatabaseError as e:
            print(e)
        finally:
            connection.close()

        return df

    def insert(self, table, schema=None, rows=None):
        """
        Insert DataFrame's rows in a database table.

        Args:
            table (obj DataFrame): DataFrame whose name and column's label
            match with table's name and column's name in database. It must be
            filled with data rows.
            schema (str): database schema
            rows (obj list): A list of row's indexes that you want to insert into database.

        Returns:
            rows_matched (int): number of rows matched.
        """
        rows_matched = 0

        connection = self.engine.connect()

        if isinstance(table, DataFrame):
            if not self.check_for_table(table.name, schema=schema):
                self.create(table, schema=schema)

            sql = "INSERT INTO "
            if schema:
                sql += "{0}.".format(schema)
            sql += "{0}".format(table.name)
            sql += ' ('
            for label in list(table):
                sql += "{0}, ".format(label)
            sql = sql[:-2]
            sql += ')'

            for index, row in table.iterrows():

                if rows is None or index in rows:
                    sql_insert = sql
                    sql_insert += " VALUES ("
                    for value in row:
                        if isinstance(value, str):
                            sql_insert += "'{0}', ".format(value)
                        else:
                            if isnull(value):
                                sql_insert += "{0}, ".format('NULL')
                            else:
                                sql_insert += "{0}, ".format(value)
                    sql_insert = sql_insert[:-2] + ')'

                rts = connection.execute(sql_insert)
                rows_matched += rts.rowcount
                rts.close()

            connection.close()

        else:
            raise TypeError("table must be a DataFrame.")

        return rows_matched

    def insert_many(self, table, schema=None):
        """
        Insert all DataFrame's rows in a database table.

        Args:
            table (obj DataFrame): DataFrame whose name and column's label
            match with table's name and column's name in database. It must be
            filled with data rows.
            schema (str): database schema

        Returns:
            rows_inserted (int): number of inserted rows.
        """
        if isinstance(table, DataFrame):
            if not self.check_for_table(table.name, schema=schema):
                self.create(table, schema=schema)

            sql_fields = "INSERT INTO "
            sql_values = " VALUES ("

            if schema:
                sql_fields += "{0}.".format(schema)
            sql_fields += "{0}".format(table.name)
            sql_fields += ' ('
            i = 0
            for label in list(table):
                i += 1
                sql_fields += "{0}, ".format(label)
                sql_values += " :" + str(i) + ","
            sql_fields = sql_fields[:-2]
            sql_values = sql_values[:-1]
            sql_fields += ")"
            sql_values += ")"

            rows = []
            for row in table.iterrows():
                index, data = row
                rows.append(data.tolist())

            try:
                cursor = self.engine.raw_connection().cursor()
                cursor.prepare(sql_fields + sql_values)
                cursor.executemany(None, rows)
                self.engine.raw_connection().commit()
                cursor.close()
                rows_inserted = rows.__len__()
            except DatabaseError as e:
                rows_inserted = 0
                self.engine.raw_connection().rollback()
                print(e)
            finally:
                self.engine.raw_connection().close()
        else:
            raise TypeError("table must be a DataFrame.")

        return rows_inserted

    def update(self, table, schema=None, index=None):
        """
        Update rows in a database table.

        Args:
            table: (obj DataFrame): DataFrame whose name and column's label
            match with table's name and columns name in database. It must
            be filled with data rows.
            schema (str): database schema.
            index: (obj list): list of DataFrame's columns names used as index
            in the update search. Other columns will be updated in database.

        Returns:
            rows_matched (int): number of rows matched.
        """
        rows_matched = 0

        connection = self.engine.connect()

        if isinstance(table, DataFrame):
            if isinstance(index, list):
                for row in table.values:
                    sql = "UPDATE "
                    if schema:
                        sql += "{0}.".format(schema)
                    sql += "{0} SET".format(table.name)
                    sql_conditions = ''
                    sql_updates = ''
                    for id, label in enumerate(table):
                        if label not in index:
                            if isinstance(row[id], str):
                                sql_updates += " {0}={1},".format(label, row[id])
                            else:
                                sql_updates += " {0}={1},".format(label, row[id])
                        else:
                            if isinstance(row[id], str):
                                sql_conditions += " {0}={1} and".format(label, row[id])
                            else:
                                sql_conditions += " {0}={1} and".format(label, row[id])
                    sql += sql_updates[:-1]

                    if len(sql_conditions) > 1:
                        sql += ' WHERE' + sql_conditions[:-4]

                    rts = connection.execute(sql)
                    rows_matched += rts.rowcount
                    rts.close()

                connection.close()
        else:
            raise TypeError("table must be a DataFrame.")

        return rows_matched

    def delete(self, table, schema=None, conditions=''):
        """
        Delete data from table.

        Args:
            table: (str): Database table name that you wish delete rows.
            schema (str): database schema
            conditions (str): optional. A string of select conditions
            to be added to WHERE clause.

        Returns:
            rows_matched (int): number of rows matched.
        """
        connection = self.engine.connect()

        sql = "DELETE FROM "
        if schema:
            sql += "{0}.".format(schema)
        sql += "{0}".format(table)

        if isinstance(conditions, str) and conditions is not '':
            sql += ' WHERE ' + conditions

        try:
            rts = connection.execute(sql)
            rows_matched = rts.rowcount
            rts.close()
        except DatabaseError as e:
            rows_matched = None
            print(e)
        finally:
            connection.close()

        return rows_matched

    @staticmethod
    def bulk_insert(user,
                    password,
                    host,
                    port,
                    service_name,
                    schema,
                    table,
                    output_path,
                    os_path,
                    os_ld_library_path,
                    mode="APPEND"):
        """
        Generates control and data files for Oracle SQL Loader by extracting field names and data values from a Pandas
        DataFrame. Requires Oracle Instant Client and Tools installed in the workstation.
        Destination table must exist in the database.
        Usage of SQL Loader:

            sqlldr <user>/<password> control=<control_file> [log=<log_file>] [bad=bad_file]

        Args:
            user (str): database user
            password (str): database password
            host: database server host name or IP address
            port (str): Oracle listener port
            service_name (str): Oracle instance service name
            schema (str): database schema
            table (pandas DataFrame): data frame whose name and column's label match with
            destination table's name and columns in database. It must be filled with data rows.
            output_path (str): path for output data files
            os_path (str): PATH environment variable
            os_ld_library_path (str): LD_LIBRARY_PATH environment variable
            mode (str): insertion mode: APPEND | REPLACE | TRUNCATE

        Returns:
            True if success or False
        """

        columns = ",".join(table.columns.values.tolist())

        # control file
        ctl_file = open(output_path + table.name + '.ctl', mode='w', encoding='utf8')
        ctl_header = """LOAD DATA\n""" + \
                     """CHARACTERSET UTF8\n""" + \
                     """INFILE '""" + output_path + table.name + '.dat' + """'\n""" + \
                     mode + """\n""" + \
                     """INTO TABLE """ + schema + """.""" + table.name + """\n""" + \
                     """FIELDS TERMINATED BY ';' OPTIONALLY ENCLOSED BY '\"'\n""" + \
                     """TRAILING NULLCOLS\n""" + \
                     """(""" + columns + """)"""
        ctl_file.write(ctl_header)
        ctl_file.close()

        # data file
        table.to_csv(output_path + table.name + '.dat',
                     sep=';',
                     header=False,
                     index=False,
                     doublequote=True,
                     quoting=csv.QUOTE_NONNUMERIC,
                     encoding='utf-8'
                     )

        # execution of Oracle SQL Loader
        os_command = "export PATH=" + os_path + "; export LD_LIBRARY_PATH=" + os_ld_library_path + "; "
        os_command += "sqlldr " + user + "/" + password + "@" + host + ":" + port + "/" + service_name + " "
        os_command += "control='" + output_path + table.name + ".ctl' "
        os_command += "log='" + output_path + table.name + ".log' "
        os_command += "bad='" + output_path + table.name + ".bad'"

        return os.system(os_command)



import re
from unidecode import unidecode


def parsed_columns(df):
    """Function to parse columns in a data frame: conversion of capital letters in lowercase letters,
    white spaces in low bar and remove accent mark.
    """
    pars_columns = []
    for column in df.columns:
        to_replace = re.sub(r'[ ]{3,}', ' ',
                            unidecode(column.lower().replace(',', '')))
        pars_columns.append(re.sub(r'[ ]{2,}', '', to_replace).replace(' ', '_'))
    df.columns = pars_columns



import csv
import fnmatch
import os
import xml.etree.ElementTree as ET
from contextlib import ExitStack

import numpy as np
import pandas as pd
from Levenshtein import ratio
from etlstat.extractor.pcaxis import *

conversion_map = {
    'STRING': str,
    'NUMBER': np.float32,
    'DECIMAL': np.float32,
    'INTEGER': np.int32
}


def data_format(dir_path, data, suffix='formato'):
    """
    Function that matches format files(csv's) with data files in a directory.

    Args:
        dir_path (str): directory containing files.
        data (str): type of data format(excel,csv,txt)
        suffix (str): standard for format name

    Returns:
        dict:   Excel name and sheet_names as KEYS and data frame as VALUE if we read excels.
                Data files names as KEYS and data frame as VALUE in other cases.
    """
    format_map = {
        'excel': '*.[xX][lL][sS]',
        'txt': '*.[tT][xX][tT]',
        'csv': '*[!' + suffix + '].[cC][sS][vV]'
    }
    # regEx for the name of format files
    format = '*' + suffix + '*.[cC][sS][vV]'
    assignation_map = {}
    # Contains data file names
    data_list = []
    # Contains format file names
    format_list = []
    os.chdir(dir_path)
    for file in os.listdir('.'):
        # Finds all format files in a directory
        if fnmatch.fnmatch(file, format):
            format_list.append(file)
        # Finds all data files in a directory
        if fnmatch.fnmatch(file, format_map[data]):
            data_list.append(file)
    format_list = set(format_list)
    format_list = list(format_list)
    if len(format_list) == 0:
        raise FileNotFoundError("Not format files found in the directory")
    data_list = set(data_list)
    data_list = list(data_list)
    for item in data_list:
        if (data == 'excel'):
            max_format = None
            max_data = None
            max_format_similarity = 0
            max_data_similarity = 0
            # I create a key for each excel
            assignation_map[item] = dict()
            assignation_map[item]['data'] = None
            assignation_map[item]['format'] = None
            # I add '_datos' and '_hoja' to data string in order to ensure a correct assignment
            data_str = item[:-4] + '_datos' + item[-4:]
            sheet_str = item[:-4] + '_hoja' + item[-4:]
            for element in format_list:
                # I check the similarity between format files and data format files
                similarity_data = ratio(data_str, element)
                # I check the similarity between format files and sheet format files
                similarity_sheet = ratio(sheet_str, element)
                if similarity_data > max_data_similarity:
                    max_data_similarity = similarity_data
                    # I take the maximum format data element
                    max_data = element
                if similarity_sheet > max_format_similarity:
                    max_format_similarity = similarity_sheet
                    # I take the maximum format sheet element
                    max_format = element
            assignation_map[item]['data'] = max_data
            assignation_map[item]['format'] = max_format
            logger.info("Matched data file: " + item + " with: " + assignation_map[item]['data'] + ' and ' +
                        assignation_map[item]['format'])

        else:
            max_similarity = 0
            max_element = None
            for element in format_list:
                similarity = ratio(item, element)
                if similarity > max_similarity:
                    max_similarity = similarity
                    max_element = element
            assignation_map[item] = max_element
            logger.info("Matched data file: " + item + " with: " + assignation_map[item])
    return assignation_map


def excel_in(dir_path, sep=';', encoding='utf-8', suffix='formato', na_values=None):
    """
    Function that reads excel files in a directory and generates a
    dict with xls names and sheetnames as keys of the dataframe.
    This method also calls data_format() in order to generate
    the data frame.

    Args:
        dir_path (str): directory containing Excel files.
        sep (str): field separator.
        encoding (str): file encoding.
        suffix (str): standard for format name.
        na_values (scalar, str, list-like, or dict) : Additional strings to recognize as NA/NaN.

    Returns:
        dict: Excel name and sheet_names as KEYS and data frame as VALUE
    """
    data = dict()
    assignation_map = data_format(dir_path, 'excel', suffix)
    for excel in assignation_map:
        data[excel] = dict()
        # I read sheet format files (csv)
        file = pd.read_csv(dir_path + assignation_map[excel]['format'], sep=sep,
                           encoding=encoding)
        for line in range(len(file)):
            # I create a key for each sheet_name (data[excel][sheet_name])
            sheetname = file['SHEET_NAME'][line]
            data[excel][sheetname] = dict()
            data[excel][sheetname]['START_ROW'] = file['START_ROW'][line]
            data[excel][sheetname]['START_COLUMN'] = file['START_COLUMN'][line]
            data[excel][sheetname]['SKIP_FOOTER'] = file['SKIP_FOOTER'][line]
            data[excel][sheetname]['FIELD_NAME'] = []
            data[excel][sheetname]['DATA_TYPE'] = dict()
        # I read data format files (csv)
        file = pd.read_csv(dir_path + assignation_map[excel]['data'], sep=sep,
                           encoding=encoding)
        # I declare FIELD NAMES and DATA_TYPE(must be a DICT)
        for line in range(len(file)):
            sheetname = file['SHEET_NAME'][line]
            data[excel][sheetname]['FIELD_NAME'].append(file['FIELD_NAME'][line])
            data[excel][sheetname]['DATA_TYPE'][str(file['FIELD_NAME'][line])] = conversion_map[
                file['DATA_TYPE'][line]]

        for sheet in data[excel]:
            data[excel][sheet] = pd.read_excel(open(dir_path + excel, 'rb'),
                                               sheet_name=str(sheet),
                                               skiprows=data[excel][sheet]['START_ROW'] - 1,
                                               skip_footer=data[excel][sheet]['SKIP_FOOTER'],
                                               names=data[excel][sheet]['FIELD_NAME'],
                                               dtype=data[excel][sheet]['DATA_TYPE'],
                                               encoding=encoding,
                                               na_values=na_values)
            data[excel][sheet].name = sheet

    return data


def csv_in(dir_path, sep=';', encoding='utf-8', suffix='formato', na_values=None):
    """
    Function that reads csv files in a directory and generates a
    dict with csv names as keys of the dataframe.
    This method also calls data_format() in order to generate
    the data frame.

    Args:
        dir_path (str): directory containing Csv files.
        sep (str): field separator
        encoding (str): file encoding
        suffix (str): standard for format name
        na_values (scalar, str, list-like, or dict) : Additional strings to recognize as NA/NaN.

    Returns:
        dict: Csv name as KEY and data frame as VALUE
    """
    assignation_map = data_format(dir_path, 'csv', suffix)
    for csv in assignation_map:
        conversion = dict()
        assignation_map[csv] = pd.read_csv(dir_path + assignation_map[csv], sep=sep,
                                encoding=encoding)
        for line in range(len(assignation_map[csv])):
            conversion[assignation_map[csv]['FIELD_NAME'][line]] = conversion_map[assignation_map[csv]['DATA_TYPE'][line]]
        assignation_map[csv] = pd.read_csv(dir_path + csv,
                                dtype=conversion,
                                encoding=encoding,
                                na_values=na_values,
                                sep=sep)
        assignation_map[csv].name = csv
    return assignation_map


def pc_axis_in(dir_path, sep=",", encoding='windows-1252'):
    """
    Reads and converts Pc-Axis files to dataframe from URIs listed
    in a CSV file.

    Args:
        dir_path (str): extractor with uris file path (including file name).
        sep (str): field separator
        encoding (str): file encoding
    Returns:
        dict: URL as KEY and dataframe as VALUE
    """
    pc_axis_dict = {}
    with open(dir_path, "rt") as f:
        reader = csv.reader(f, delimiter=sep)
        for row in reader:
            return_dict = from_pc_axis(row[1], encoding)
            pc_axis_dict[row[0]] = return_dict['DATA']

    return pc_axis_dict


def positional_in(dir_path, sep=';', encoding='windows-1252', suffix='', na_values=None):
    """
    Function that reads files in a directory, generates a correspondence
    between data files and format files and returning a dict. (MICRODATA).
    This method also calls data_format() in order to generate
    the data frame.

    Args:
        dir_path (str): directory containing Positional files.
        sep (str): field separator.
        encoding (str): file encoding.
        suffix (str): standard for format name
        na_values (scalar, str, list-like, or dict) : Additional strings to recognize as NA/NaN.


    Returns:
        dict: Name of data file as KEY and dataframe as VALUE
    """
    assignation_map = data_format(dir_path, 'txt', suffix)
    for txt in assignation_map:
        conversion = dict()
        assignation_map[txt] = pd.read_csv(dir_path + assignation_map[txt], sep=sep,
                                encoding=encoding)
        for line in range(len(assignation_map[txt])):
            conversion[assignation_map[txt]['FIELD_NAME'][line]] = conversion_map[assignation_map[txt]['DATA_TYPE'][line]]
        assignation_map[txt] = pd.read_fwf(dir_path + txt,
                                widths=assignation_map[txt]['LENGTH'].tolist(),
                                names=assignation_map[txt]['FIELD_NAME'].tolist(),
                                dtype=conversion,
                                nwords=0,
                                encoding=encoding,
                                na_values=na_values)
        assignation_map[txt].name = txt
    return assignation_map


def xml_in(dir_path, pattern='*.[xXkK][mMtTjJ][lLrRbB]'):
    """
    Function that reads files in a directory filtered by regEx and generates a
    dict with xml names.

    Args:
        dir_path (str): directory containing XML files.
        pattern (str): regEx to filter data names

    Returns:
        dict: XML name as KEY and etree object as VALUE
    """
    xml_list = []
    os.chdir(dir_path)
    for file in os.listdir('.'):
        if fnmatch.fnmatch(file, pattern):
            xml_list.append(file)
    xml_list = set(xml_list)
    xml_list = list(xml_list)
    df_dict = dict.fromkeys(xml_list, '')
    for i in range(len(xml_list)):
        df_dict[xml_list[i]] = ET.parse(dir_path + xml_list[i])

    return df_dict


def sql_in(dir_path):
    """
    Function that open sql files in the sql directory using a context manager
    Args:
        dir_path (str): Path of DIR readed.

    Returns:
        dict: query name  as KEY and query as VALUE
    """
    files = {}
    os.chdir(dir_path)
    with ExitStack() as cm:
        for filename in os.listdir('.'):
            if fnmatch.fnmatch(filename, '*.sql'):
                f = cm.enter_context(open(filename, 'r'))
                files[filename.split(dir_path, 1)[-1][:-4]] = f.read()
        cm.pop_all().close()

    return files



import unittest
import pandas
from etlstat.text.parsed_columns import parsed_columns


class TestParsedColumns(unittest.TestCase):

    def testParsedColumns(self):
        df = pandas.DataFrame
        df.columns = ['Período', 'Clasificación CNAE 2009', 'Número de trabajadores', 'Índice de variación']
        parsed_columns(df)
        self.assertEqual(df.columns[0], 'periodo')
        self.assertEqual(df.columns[1], 'clasificacion_cnae_2009')
        self.assertEqual(df.columns[2], 'numero_de_trabajadores')
        self.assertEqual(df.columns[3], 'indice_de_variacion')

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

# database
from etlstat.database.mysql import MySQL
from etlstat.database.oracle import Oracle
from etlstat.database.simplesql import SimpleSQL

# extractor
from etlstat.extractor.extractor import excel_in, csv_in, pc_axis_in, \
                                        positional_in, xml_in, sql_in
from etlstat.extractor.pcaxis import read, from_pc_axis

# log
from etlstat.log.timing import timeit

# text
from etlstat.text.parsed_columns import parsed_columns





import unittest
import pandas as pd
from etlstat.database.mysql import MySQL


class TestMySQL(unittest.TestCase):

    user = 'test'
    password = 'password'
    host = '127.0.0.1'
    port = '3307'
    database = 'test'
    conn_params = [user, password, host, port, database]

    @classmethod
    def setUpClass(cls):
        user = 'root'
        password = 'password'
        host = '127.0.0.1'
        port = '3307'
        database = ''
        conn_params = [user, password, host, port, database]
        ddl = "DROP DATABASE test"
        my_conn = MySQL(*conn_params)
        status, result = my_conn.execute_sql(ddl)
        print(result)
        ddl = "CREATE DATABASE test"
        status, result = my_conn.execute_sql(ddl)
        print(result)
        sql = "CREATE TABLE test.inf_schema as " \
              "SELECT table_name, avg_row_length, table_collation, create_time " \
              "FROM information_schema.tables"
        status, result = my_conn.execute_sql(sql)
        print(result)
        ddl = "GRANT ALL PRIVILEGES ON test.* to 'test'@'%' IDENTIFIED BY 'password'"
        status, result = my_conn.execute_sql(ddl)
        print(result)

    def testInit(self):
        self.assertEqual(str(MySQL(*self.conn_params).engine),
                         "Engine(mysql+mysqlconnector://test:***@127.0.0.1:3307/test)")

    def testCheckForTable(self):
        my_conn = MySQL(*self.conn_params)
        self.assertTrue(my_conn.check_for_table('inf_schema'))

    def testExecuteSql(self):
        my_conn = MySQL(*self.conn_params)
        sql = "CREATE TABLE table1 (id integer, column1 varchar(100), column2 double)"
        status, result = my_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = "INSERT INTO table1 (id, column1, column2) " \
              "VALUES (1, 'Varchar text (100 char)', 123456789.0123456789)"
        status, result = my_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = 'select * from table1 order by id'
        status, result = my_conn.execute_sql(sql)
        self.assertEqual(result.columns.values[0], 'id')
        self.assertEqual(result.columns.values[1], 'column1')
        self.assertEqual(result.columns.values[2], 'column2')
        self.assertEqual(result.values[0][0], 1)
        self.assertEqual(result.values[0][1], 'Varchar text (100 char)')
        self.assertEqual(result.values[0][2], 123456789.0123456789)
        sql = "UPDATE table1 SET id = 2 WHERE id = 1"
        status, result = my_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = "DELETE FROM table1 WHERE id > 0"
        status, result = my_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = "DROP TABLE table1"
        status, result = my_conn.execute_sql(sql)
        self.assertTrue(status)

    def testCreate(self):
        # Create data frame
        data_columns = ['column_int', 'column_string', 'column_float']
        table = pd.DataFrame(columns=data_columns)

        # Assign columns data types
        table['column_int'] = table['column_int'].astype(int)
        table['column_string'] = table['column_string'].astype(str)
        table['column_float'] = table['column_float'].astype(float)

        # Rename data frame
        table.name = "table2"
        my_conn = MySQL(*self.conn_params)
        self.assertTrue(my_conn.create(table))

    def testSelect(self):
        my_conn = MySQL(*self.conn_params)
        table_name = "inf_schema"
        conditions = "table_name like 'INNO%' AND avg_row_length > 100"
        table_df = my_conn.select(table_name, conditions=conditions)
        self.assertGreaterEqual(len(table_df), 6)

    def testInsert(self):
        data_columns = ['column_int', 'column_string', 'column_float']
        data_values = [[1, 'string1', 456.956], [2, 'string2', 38.905]]
        df = pd.DataFrame(data_values, columns=data_columns)
        df.name = 'test_insert'
        my_conn = MySQL(*self.conn_params)
        self.assertEqual(my_conn.insert(df), 2)

    def testUpdate(self):
        data_columns = ['column_int', 'column_string', 'column_float']
        data_values1 = [[1, 'string1', 456.956], [2, 'string2', 38.905]]
        df1 = pd.DataFrame(data_values1, columns=data_columns)
        df1.name = 'test_update'
        my_conn = MySQL(*self.conn_params)
        my_conn.insert(df1)
        data_values2 = [[1, "updated string1", 456.956], [2, "updated string2", 38.905]]
        df2 = pd.DataFrame(data_values2, columns=data_columns)
        df2.name = 'test_update'
        self.assertEqual(my_conn.update(df2, index=['column_int']), 2)

    def testDelete(self):
        data_columns = ['column_int', 'column_string', 'column_float']
        data_values = [[1, 'string1', 456.956], [2, 'string2', 38.905]]
        df = pd.DataFrame(data_values, columns=data_columns)
        df.name = 'test_delete'
        my_conn = MySQL(*self.conn_params)
        my_conn.insert(df)
        self.assertEqual(my_conn.delete(df.name, conditions="column_int = 2"), 1)

    def testBulkInsert(self):
        data_columns = ['id', 'column_string', 'column_float']
        data_types = {'id': int, 'column_string': object, 'column_float': float}
        data_values = [
            [1, 'Alpha',	5.39282029181928],
            [2,	'Beta',	62.0512266797524],
            [3, 'Gamma', 7.20169799112829],
            [4, 'Delta', 17.2716470303442]
        ]
        df = pd.DataFrame(data=data_values, columns=data_columns)
        for k, v in data_types.items():
            df[k] = df[k].astype(v)

        df.name = 'test_bulk_insert'
        my_conn = MySQL(*self.conn_params)
        self.assertEqual(my_conn.bulk_insert(df,
                                             csv_path='test_bulk_insert.csv', sep=';', header=False, index=False), 4)

if __name__ == '__main__':
    unittest.main()



import string
from random import *


def generate_password(
        min_length,
        max_length,
        first_letter=False,
        upper_case=False,
        lower_case=False,
        punctuation=True,
        digits=True
):
    """
    Generates a random password.
    :param min_length: (:int:): password min. length
    :param max_length: (:int:): password max. length
    :param first_letter: (:bool:): True if first character must be a letter
    :param upper_case: (:bool:): True if only uppercase letters
    :param lower_case: (:bool:): True if only lowercase letters
    :param punctuation: (:bool:): by default, use punctuation signs
    :param digits: (:bool:): by default, use digits
    :return: (:obj:str:): generated password
    """

    if min_length > max_length:
        raise TypeError("Minimum password length must be less or equal than maximum.")

    if upper_case and not lower_case:
        letters = string.ascii_uppercase
    elif lower_case and not upper_case:
        letters = string.ascii_lowercase
    else:
        letters = string.ascii_letters

    if first_letter:
        first = choice(letters)
    else:
        first = ''

    valid_characters = letters
    if punctuation:
        valid_characters += string.punctuation

    if digits:
        valid_characters += string.digits

    password = first + ''.join(choice(valid_characters) for x in range(randint(min_length - 1, max_length - 1)))
    return password



import unittest
from etlstat.database.oracle import Oracle
import os
import pandas as pd


class TestOracle(unittest.TestCase):

    output_path = './'
    os_path = '/usr/local/bin:/usr/bin:/bin:/opt/oracle/instantclient_12_2'
    os_ld_library_path = '/opt/oracle/instantclient_12_2'
    schema = 'test'
    user = 'test'
    password = 'test'
    host = 'localhost'
    port = '1521'
    service_name = 'xe'
    conn_params = [user, password, host, port, service_name]

    @classmethod
    def setUpClass(cls):
        user = 'system'
        password = 'oracle'
        host = 'localhost'
        port = '1521'
        service_name = 'xe'
        conn_params = [user, password, host, port, service_name]
        ora_conn = Oracle(*conn_params)
        sql = "DROP USER TEST CASCADE"
        status, result = ora_conn.execute_sql(sql)
        print(result)
        sql = "CREATE USER test IDENTIFIED BY test " \
              "DEFAULT TABLESPACE USERS TEMPORARY TABLESPACE TEMP"
        status, result = ora_conn.execute_sql(sql)
        print(result)
        sql = "ALTER USER test QUOTA UNLIMITED ON USERS"
        status, result = ora_conn.execute_sql(sql)
        print(result)
        sql = "GRANT CONNECT, RESOURCE TO test"
        status, result = ora_conn.execute_sql(sql)
        print(result)

    def testInit(self):
        self.assertEqual(str(Oracle(*self.conn_params).engine),
                         "Engine(oracle+cx_oracle://test:***@localhost:1521/?service_name=xe)")

    def testCheckForTable(self):
        ora_conn = Oracle(*self.conn_params)
        sql = "CREATE TABLE test_check (id integer, column1 varchar2(100), column2 number)"
        ora_conn.execute_sql(sql)
        self.assertTrue(ora_conn.check_for_table('test_check', schema='test'))
        sql = "DROP TABLE test_check"
        ora_conn.execute_sql(sql)

    def testExecuteSql(self):
        ora_conn = Oracle(*self.conn_params)
        sql = "CREATE TABLE test_sql (id integer, column1 varchar2(100), column2 number)"
        status, result = ora_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = "INSERT INTO test_sql (id, column1, column2) " \
              "VALUES (1, 'Varchar text (100 char)', 123456789.012787648484859)"
        status, result = ora_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = 'select * from test_sql order by id'
        status, result = ora_conn.execute_sql(sql)
        self.assertEqual(result.columns.values[0], 'id')
        self.assertEqual(result.columns.values[1], 'column1')
        self.assertEqual(result.columns.values[2], 'column2')
        self.assertEqual(result.values[0][0], 1)
        self.assertEqual(result.values[0][1], 'Varchar text (100 char)')
        self.assertEqual(result.values[0][2], 123456789.012787648484859)
        sql = "UPDATE test_sql SET id = 2 WHERE id = 1"
        status, result = ora_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = "DELETE FROM test_sql WHERE id > 0"
        status, result = ora_conn.execute_sql(sql)
        self.assertTrue(status)
        sql = "DROP TABLE test_sql"
        status, result = ora_conn.execute_sql(sql)
        self.assertTrue(status)

    def testCreate(self):
        # Create data frame
        data_columns = ['column_int', 'column_string', 'column_float']
        table = pd.DataFrame(columns=data_columns)

        # Assign columns data types
        table['column_int'] = table['column_int'].astype(int)
        table['column_string'] = table['column_string'].astype(str)
        table['column_float'] = table['column_float'].astype(float)

        # Rename data frame
        table.name = "test_create"
        ora_conn = Oracle(*self.conn_params)
        self.assertTrue(ora_conn.create(table, 'test'))
        sql = "DROP TABLE test_create"
        ora_conn.execute_sql(sql)

    def testSelect(self):
        ora_conn = Oracle(*self.conn_params)
        table_name = "all_tables"
        conditions = "owner = 'CTXSYS'"
        df = ora_conn.select(table_name, conditions=conditions)
        self.assertEqual(len(df), 5)

    def testInsert(self):
        data_columns = ['column_int', 'column_string', 'column_float']
        data_values1 = [[1, 'stríng1', 456.956], [2, 'string2', 38.905]]
        df = pd.DataFrame(data_values1, columns=data_columns)
        df.name = 'test_insert'
        ora_conn = Oracle(*self.conn_params)
        self.assertEqual(ora_conn.insert(df), 2)
        sql = "DROP TABLE test_insert"
        ora_conn.execute_sql(sql)

    def testInsertMany(self):
        df = pd.read_csv('test_insert.csv', header=0, sep=',', encoding='utf8')
        df.name = 'test_insert'
        ora_conn = Oracle(*self.conn_params)
        self.assertEqual(ora_conn.insert_many(df, 'test'), 2)
        sql = "DROP TABLE test_insert"
        ora_conn.execute_sql(sql)

    def testUpdate(self):
        data_columns = ['column_int', 'column_string', 'column_float']
        data_values1 = [[1, 'string1', 456.956], [2, 'string2', 38.905]]
        df1 = pd.DataFrame(data_values1, columns=data_columns)
        df1.name = 'test_update'
        ora_conn = Oracle(*self.conn_params)
        ora_conn.insert(df1)
        data_values2 = [[1, "'updated string1'", 456.956], [2, "'updated string2'", 38.905]]
        df2 = pd.DataFrame(data_values2, columns=data_columns)
        df2.name = 'test_update'
        self.assertEqual(ora_conn.update(df2, index=['column_int']), 2)
        sql = "DROP TABLE test_update"
        ora_conn.execute_sql(sql)

    def testDelete(self):
        data_columns = ['column_int', 'column_string', 'column_float']
        data_values = [[1, 'string1', 456.956], [2, 'string2', 38.905]]
        df = pd.DataFrame(data_values, columns=data_columns)
        df.name = 'test_delete'
        ora_conn = Oracle(*self.conn_params)
        ora_conn.insert(df)
        self.assertEqual(ora_conn.delete(df.name, conditions="column_int = 2"), 1)
        sql = "DROP TABLE test_delete"
        ora_conn.execute_sql(sql)
        
    def testBulkInsert(self):
        source_file = '01001.csv'
        table_name = 'px_01001'
        data_file = 'px_01001.dat'
        control_file = 'px_01001.ctl'
        log_file = 'px_01001.log'
        bad_file = 'px_01001.bad'
        data_columns = ['id', 'tipo_indicador', 'nivel_educativo', 'valor']
        table_def = pd.DataFrame(columns=data_columns)
        table_def['id'] = table_def['id'].astype(int)
        table_def['tipo_indicador'] = table_def['tipo_indicador'].astype(str)
        table_def['nivel_educativo'] = table_def['nivel_educativo'].astype(str)
        table_def['valor'] = table_def['valor'].astype(float)
        table_def.name = table_name
        ora_conn = Oracle(*self.conn_params)
        ora_conn.create(table_def, self.schema)

        df = pd.read_csv(source_file, header=0, sep=';', encoding='utf8')
        df.name = table_name
        ora_conn.bulk_insert(self.user,
                             self.password,
                             self.host,
                             self.port,
                             self.service_name,
                             self.schema,
                             df,
                             self.output_path,
                             self.os_path,
                             self.os_ld_library_path,
                             mode="TRUNCATE")
        self.assertTrue(os.path.isfile(data_file))
        self.assertTrue(os.path.isfile(control_file))
        self.assertTrue(os.path.isfile(log_file))
        self.assertFalse(os.path.isfile(bad_file))

if __name__ == '__main__':
    unittest.main()



from etlstat.database.oracle import Oracle
import pandas as pd

schema = 'test'
user = 'test'
password = 'test'
host = 'localhost'
port = '1521'
service_name = 'xe'
conn_params = [user, password, host, port, service_name]

data_columns = ['column_int', 'column_string', 'column_float']
data_values1 = [[1, 'stríng1', 456.956], [2, 'string2', 38.905]]
df = pd.DataFrame(data_values1, columns=data_columns)
df.name = 'test_insert'
ora_conn = Oracle(*conn_params)
ora_conn.insert(df)



import unittest
import pandas as pd

from icaneconfig import Configuration, Env
from etlstat.database.simplesql import SimpleSQL
from etlstat.database.mysql import MySQL


class TestMySQL(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        config_global = {
            'mysql': {
                'connector': 'mysql+mysqlconnector',
                'local': {
                    'user': 'root',
                    'password': 'password',
                    'host': '127.0.0.1',
                    'port': '3307'
                },
                'dev': {
                    'user': 'root',
                    'password': 'password',
                    'host': '127.0.0.1',
                    'port': '3307'
                },
                'pro': {
                    'user': 'root',
                    'password': 'password',
                    'host': '127.0.0.1',
                    'port': '3307'
                }
            },
            'store': ''
        }
        cls.config = Configuration(config_global)

        config_local = {
            'database': 'banco_series',
            'cubes': {
                'ind_gen_destino_econ': {
                    'tables': {
                        'table_agg_84': 'agg_1_px_3284',
                        'table_84': 'px_3284'
                    }
                },
                'ind_sec_div': {
                    'tables': {
                        'table_80': 'agg_1_px_3281',
                        'table_81': 'px_3281'
                    }
                }
            }
        }
        cls.config.add(config_local)
        cls.config.environment = Env.LOCAL

        user = 'root'
        password = 'password'
        host = '127.0.0.1'
        port = '3307'
        database = ''
        conn_params = [user, password, host, port, database]

        ddl = "DROP DATABASE banco_series"
        my_conn = MySQL(*conn_params)
        my_conn.execute_sql(ddl)

        ddl = "CREATE DATABASE banco_series"
        my_conn.execute_sql(ddl)

        sql = "CREATE TABLE test.inf_schema as " \
              "SELECT table_name, avg_row_length, table_collation, create_time " \
              "FROM information_schema.tables"
        my_conn.execute_sql(sql)

        ddl = "GRANT ALL PRIVILEGES ON test.* to 'test'@'%' IDENTIFIED BY 'password'"
        my_conn.execute_sql(ddl)

    def testone(self):
        SimpleSQL.drop(TestMySQL.config.cubes.ind_gen_destino_econ.tables.table_agg_84, TestMySQL.config)

        df = SimpleSQL.pull(TestMySQL.config.cubes.ind_gen_destino_econ.tables.table_agg_84, TestMySQL.config)

        self.assertEqual(df, None)

        data_columns = ['column_int', 'column_string', 'column_float']
        data_values = [[56464, 'string1', 456.956], [23, 'string2', 38.905]]
        df = pd.DataFrame(data_values, columns=data_columns)
        df.name = TestMySQL.config.cubes.ind_gen_destino_econ.tables.table_agg_84
        SimpleSQL.push(df, TestMySQL.config)

        df = pd.DataFrame()
        df.name = TestMySQL.config.cubes.ind_gen_destino_econ.tables.table_agg_84
        rst = SimpleSQL.pull(df, TestMySQL.config)

        self.assertEqual(len(rst), 2)

        SimpleSQL.drop(TestMySQL.config.cubes.ind_gen_destino_econ.tables.table_agg_84, TestMySQL.config)



import unittest
from etlstat.extractor.extractor import *
import config_test as ct


class TestExtractor(unittest.TestCase):

    base_path = ct.GLOBAL_PATH + '/etlstat/extractor/test/extractor_test_files/'

    def testSimilar(self):
        string1 = "HOLA"
        string2 = "ADIOS"
        string3 = "HOLA MUNDO"
        self.assertGreater(ratio(string1, string3), ratio(string1, string2))

        string1 = "IPI_JUNIO_2017.txt"
        string2 = "JUNIO_IPI_2017.csv"
        string3 = "IPI_Junio_2016.csv"
        self.assertGreater(ratio(string1, string2), ratio(string1, string3))

        string1 = "EEE_2015_TERRI_IND_06.txt"
        string2 = "EEE_TERRI.csv"
        string3 = "EEE_UA.csv"
        self.assertGreater(ratio(string1, string2), ratio(string1, string3))

        string1 = 'EEE_2015_IDENT_IND_06.TXT'
        string2 = 'EEE_FINAL.csv'
        string3 = 'EEE_IDENT.csv'
        self.assertGreater(ratio(string1, string3), ratio(string1, string2))



    def testCsvIn(self):
        dir_path = self.base_path + 'csv/'
        df = csv_in(dir_path, sep=";")
        log.debug(df.keys())
        agenv_head = ['NORDEN', 'QI11', 'QI121', 'QI122', 'QI131', 'QI132', 'QI1331',
       'QI1332', 'QI1333', 'QI1334', 'QI134', 'QI135', 'QI135E', 'QI14',
       'QI15', 'QI15E', 'QI21', 'QI221', 'QI222', 'QI231', 'QI232', 'QI24',
       'QI25', 'QJ11', 'QJ12', 'QJ131', 'QJ132', 'QJ133', 'QJ134', 'QJ14',
       'QJ15', 'QJ16', 'QJ17', 'QJ18', 'QJ18E', 'QJ2', 'QJ3', 'QJ4', 'QJ5',
       'QJ6', 'QJ6E']

        aereo_head = ['NORDEN', 'AI1', 'AI11', 'AI12', 'AI2', 'AI21', 'AI22', 'AI3', 'AI4',
       'AI5', 'AJ1', 'AJ2', 'AJ3', 'AJ4', 'AJ5', 'AJ6', 'AJ7', 'AJ7E', 'AK111',
       'AK121', 'AK131', 'AK112', 'AK122', 'AK132', 'AK211', 'AK221', 'AK231',
       'AK212', 'AK222', 'AK232', 'AK213', 'AK223', 'AK233', 'AK214', 'AK224',
       'AK234', 'AM1', 'AM11', 'AM2']

        aloj_head = ['NORDEN', 'HI11', 'HI12', 'HI13', 'HI14', 'HI15', 'HI16', 'HI17',
       'HI18', 'HI19', 'HI19E', 'HI21', 'HI221', 'HI222', 'HI231', 'HI232',
       'HI24', 'HI25', 'HI25E', 'HJ1', 'HJ2', 'HJ3', 'HJ4', 'HJ5', 'HJ6',
       'HJ7', 'HJ8', 'HJ9', 'HJ10', 'HJ10E']

        aereo_type = ['int32', 'int32', 'float32', 'float32', 'float32', 'float32', 'int32',
        'int32', 'int32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'float32', 'float32', 'O', 'float32', 'float32', 'float32', 'float32', 'float32',
        'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32']

        agenv_type = ['int32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'O', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'O']

        aloj_type = ['int32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'float32', 'float32', 'O', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
        'O', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'O']



        assert (df['AEREO_SER_06_15.csv'].shape[0] == 2)
        assert (df['AEREO_SER_06_15.csv'].shape[1] == 39)
        assert (df['AGENV_SER_06_15.csv'].shape[0] == 7)
        assert (df['AGENV_SER_06_15.csv'].shape[1] == 41)
        assert (df['ALOJ_SER_06_15.csv'].shape[0] == 11)
        assert (df['ALOJ_SER_06_15.csv'].shape[1] == 30)
        assert (list(df['AEREO_SER_06_15.csv'].columns) == aereo_head)
        assert (list(df['AGENV_SER_06_15.csv'].columns) == agenv_head)
        assert (list(df['ALOJ_SER_06_15.csv'].columns) == aloj_head)
        assert (list(df['AEREO_SER_06_15.csv'].dtypes) == aereo_type)
        assert (list(df['AGENV_SER_06_15.csv'].dtypes) == agenv_type)
        assert (list(df['ALOJ_SER_06_15.csv'].dtypes) == aloj_type)

    def testPositionalIn(self):
        dir_path = self.base_path + 'positional/'
        df = positional_in(dir_path)

        post_head = ['NORDEN', 'PI1', 'PI2', 'PI2R', 'PI2N', 'PI21', 'PI31', 'PI41', 'PI42',
                     'PJ1', 'PJ11', 'PJ12', 'PJ13', 'PJ2', 'PJ21', 'PJ22', 'PJ3', 'PJ4',
                     'PJ5', 'PK11', 'PK121', 'PK122', 'PK13', 'PK14', 'PK15', 'PK16', 'PK17',
                     'PK17E', 'PK21', 'PK22', 'PK23', 'PK31', 'PK32', 'PK33', 'PK34', 'PK35',
                     'PK36', 'PK37', 'PK38', 'PK39', 'PK310', 'PK311', 'PK312', 'PK313',
                     'PK314', 'PK315', 'PK316', 'PK316E', 'PL11', 'PL12', 'PL13', 'PL14',
                     'PL21', 'PL22', 'PL23', 'PL24', 'PL31', 'PL32', 'PL33', 'PL34', 'PL41',
                     'PL42', 'PL43', 'PL44', 'PL51', 'PL52', 'PL53', 'PL54', 'PL5E', 'PM1',
                     'PM21', 'PM22', 'PM3', 'PM4']

        post_type = ['O', 'float32', 'float32', 'O', 'O', 'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32','float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'O',
                     'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'O', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32','float32', 'float32', 'float32','float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'O', 'float32', 'float32', 'float32', 'float32',
                     'float32']

        tec_head = ['NORDEN', 'KI111', 'KI112', 'KI113', 'KI114', 'KI115', 'KI116', 'KI121',
                    'KI122', 'KI123', 'KI124', 'KI125', 'KI126', 'KI127', 'KI128', 'KI129',
                    'I129E', 'KI131', 'KI132', 'KI133', 'KI134', 'KI134E', 'KI141', 'KI142',
                    'KI143', 'KI144', 'KI145', 'KI145E', 'KI15', 'KI16', 'KI16E', 'KI21',
                    'KI22', 'KI23', 'KI24', 'KI25', 'KI26', 'KI27', 'KI28', 'KI29', 'KI210',
                    'KI211', 'KI212', 'KI212E']

        tec_type = ['O', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
                    'float32', 'float32','float32', 'float32', 'float32', 'float32', 'float32', 'O', 'float32',
                    'float32', 'float32', 'float32', 'O', 'float32', 'float32','float32', 'float32', 'float32', 'O',
                    'float32', 'float32', 'O', 'float32', 'float32', 'float32', 'float32', 'float32', 'float32',
                    'float32', 'float32', 'float32', 'float32', 'float32', 'float32', 'O']

        assert (df['AEREO_SER_06_15.TXT'].shape[0] == 2)
        assert (df['AEREO_SER_06_15.TXT'].shape[1] == 39)
        assert (df['AGENV_SER_06_15.TXT'].shape[0] == 7)
        assert (df['AGENV_SER_06_15.TXT'].shape[1] == 41)
        assert (df['ALOJ_SER_06_15.TXT'].shape[0] == 11)
        assert (df['ALOJ_SER_06_15.TXT'].shape[1] == 30)
        assert (list(df['POST_SER_06_15.TXT'].columns)) == post_head
        assert (list(df['POST_SER_06_15.TXT'].dtypes)) == post_type
        assert (list(df['TEC_SER_06_15.TXT'].columns)) == tec_head
        assert (list(df['TEC_SER_06_15.TXT'].dtypes)) == tec_type

    def testExcelIn(self):
        dir_path = self.base_path + 'excel/'
        df = excel_in(dir_path)

        assert (len(df['Datos_web.xls']) == 6)
        assert (len(df['3 Carga_web.xls']) == 6)
        assert (type(df['3 Carga_web.xls'][2]) == pandas.core.frame.DataFrame)
        assert (type(df['Datos_web.xls'][2]) == pandas.core.frame.DataFrame)
        assert (type(df['Datos_web.xls'][3]) == pandas.core.frame.DataFrame)
        assert (type(df['Datos_web.xls'][4]) == pandas.core.frame.DataFrame)
        assert (type(df['Datos_web.xls'][5]) == pandas.core.frame.DataFrame)
        assert (type(df['Datos_web.xls'][6]) == pandas.core.frame.DataFrame)


    def testXmlIn(self):
        xml_dict = xml_in(self.base_path + 'xml/')
        assert (len(xml_dict) == 11)
        root = xml_dict['Comex.kjb'].getroot()
        assert (root.tag == 'job')
        root = xml_dict['Ec_SE_IEFAZ.ktr'].getroot()
        assert (root.tag == 'transformation')

    def testSqlIn(self):
        sql_data = sql_in(self.base_path + 'sql/')
        self.assertEqual(len(sql_data['afiliados']), 1582)
        self.assertEqual(len(sql_data['contratos']), 3023)














# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='etlstat',
    version='0.1.3',
    author='Instituto Cántabro de Estadística',
    author_email='icane.datos@cantabria.es',
    packages=find_packages(),
    url='https://github.com/icane/etlstat.git',
    download_url='https://github.com/icane/etlstat/archive/0.1.3.tar.gz',
    license='Apache License 2.0',
    description='Python package that contains extract-transform-load utils for statistical offices data processing',
    long_description=open('README.md').read(),
    install_requires=['requests'],
    keywords=['etl', 'icane', 'statistics', 'utils'],
    classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: Apache Software License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Topic :: Scientific/Engineering :: Information Analysis',
          'Topic :: Software Development :: Libraries'
          ],
)



"""
    Class to handle time logging

    Date:
        September 2017

    Authors:
        emm13775

    Version:
        0.1

    Notes:

"""

import time
import logging as log

log.basicConfig(level=log.INFO)
logger = log.getLogger(__name__)


def timeit(method):

    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()

        logger.info('[time] method: %r (%r, %r) %2.2f sec' % (method.__name__, args, kw, te-ts))
        return result

    return timed
